#!/usr/bin/ruby -w

#
# cvprof_report.rb
# $Revision: 1.4 $
#
# Copyright (c) 2004 David Holroyd
# See the file COPYING for license terms
#
# http://www.badgers-in-foil.co.uk/projects/cvprof/
#

require 'bdb'
require 'getoptlong'
require 'htmlreport'
require 'cvprof'

$version = "0.1.1"


def process_args(reporter)
  args = Array.new
  args << ["--sourcepath", "-s", GetoptLong::REQUIRED_ARGUMENT]

  reporter.add_args(args)

  opts = GetoptLong.new(*args)

  spath = "."

  opts.each do |opt, arg|
    spath = arg if opt=="--sourcepath"
  end

  reporter.process_args(opts)

  $source_path = SourcePath.new(spath)
end



def load_class_list(db)
  list = Array.new
  db.each do |key, value|
    if key =~ /(.*):source$/
      jclass = JClassInfo.new($1, value)
      # add to array rather then yield, as db access parallel with this
      # iteraton is likely not legal, so don't allow for the possibility
      list << jclass
    end
  end
  list
end


def create_package_list(classes)
  packages = Hash.new
  classes.each do |jclass|
    jpackage = packages[jclass.package_name]
    if jpackage.nil?
      jpackage = JPackage.new(jclass.package_name)
      packages[jpackage.name] = jpackage
    end
    jpackage.add_class(jclass)
  end
  packages.values.sort do |a, b|
    a.name <=> b.name
  end
end



def read_database
  $db = BDB::Hash.open $dbname, nil, BDB::RDONLY

  classes = load_class_list($db)

  yield create_package_list(classes)

  $db.close
end


$dbname = "coverage.db"

report = HtmlReport.new

process_args(report)

read_database do |packages|
  report.generate(packages)
end

