/*
 * cvprof_test.c
 * $Revision: 1.4 $
 *
 * Copyright (c) 2004 David Holroyd
 * See the file COPYING for license terms
 *
 * http://www.badgers-in-foil.co.uk/projects/cvprof/
 */


#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#define NO_JVMDI_MACROS
#include <jvmdi.h>
#include <jni.h>


/*
 * This is a test harness for cvprof that lets us run cvprof under valgrind.
 * (Running the whole JVM under valgrind didn't appear to work, repeatedly
 * printing out the Memcheck welcome message.)
 *
 * We implement just enough of the JVMDI interface that most of the
 * functionality of cvprof gets exercised.
 */


extern jint JVM_OnLoad(JavaVM *jvm, void *options, void *reserved);

JVMDI_Interface_1 *my_jvmdi;

JVMDI_EventHook my_jvmdi_hook;

/* lies, tbh */
#define MYCLASS ((jclass)0xCAFEBABE)
#define MYMETHOD ((jmethodID)0xDEADBEEF)


jint jvm_get_env(JavaVM *vm, void **penv, jint version)
{
	if (version == JVMDI_VERSION_1) {
		(*penv) = my_jvmdi;
		return JVMDI_ERROR_NONE;
	}
	return JVMDI_ERROR_ILLEGAL_ARGUMENT;
}


jvmdiError jvmdi_get_caps(JVMDI_capabilities *capabilitiesPtr)
{
	capabilitiesPtr->can_get_synthetic_attribute = 1;
	capabilitiesPtr->can_get_bytecodes = 0;
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_set_event_notify(jint mode, jint eventType, jthread thread,
                                  ...)
{
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_set_event_hook(JVMDI_EventHook hook)
{
	my_jvmdi_hook = hook;
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_create_raw_monitor(char *name, JVMDI_RawMonitor *monitorPtr)
{
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_destroy_raw_monitor(JVMDI_RawMonitor monitorPtr)
{
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_get_class_sig(jclass clazz, char **sigPtr)
{
	char *sig = "Lpakg.Clss;";
	if (clazz == MYCLASS) {
		*sigPtr = malloc(strlen(sig)+1);
		strcpy(*sigPtr, sig);
		return JVMDI_ERROR_NONE;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_deallocate(jbyte* mem)
{
	free(mem);
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_raw_monitor_enter(JVMDI_RawMonitor monitor)
{
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_raw_monitor_exit(JVMDI_RawMonitor monitor)
{
	return JVMDI_ERROR_NONE;
}


jvmdiError jvmdi_get_class_methods(jclass clazz, jint *methodCountPtr,
                                   jmethodID **methodsPtr)
{
	if (clazz == MYCLASS) {
		*methodCountPtr = 1;
		*methodsPtr = malloc(sizeof(jmethodID)*1);
		(*methodsPtr)[0] = MYMETHOD;
		return JVMDI_ERROR_NONE;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_is_method_synthetic(jclass clazz, jmethodID method,
                                     jboolean *isSyntheticPtr)
{
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*isSyntheticPtr = JNI_FALSE;
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_is_method_native(jclass clazz, jmethodID method,
                                  jboolean *isNativePtr)
{
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*isNativePtr = JNI_FALSE;
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_get_source_file_name(jclass clazz, char **sourceNamePtr)
{
	char *src = "Clss.java";
	if (clazz == MYCLASS) {
		*sourceNamePtr = malloc(strlen(src)+1);
		strcpy(*sourceNamePtr, src);
		return JVMDI_ERROR_NONE;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_line_number_table(jclass clazz, jmethodID method,
                                   jint *entryCountPtr,
                                   JVMDI_line_number_entry **tablePtr)
{
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*entryCountPtr = 1;
			*tablePtr = (JVMDI_line_number_entry *)
				malloc(sizeof(JVMDI_line_number_entry)*1);
			(*tablePtr)->start_location = 0;
			(*tablePtr)->line_number = 1;
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_get_method_name(jclass clazz, jmethodID method, 
                                 char **namePtr, char **signaturePtr)
{
	char *name = "foo";
	char *sig = "()V";
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*namePtr = malloc(strlen(name)+1);
			strcpy(*namePtr, name);
			*signaturePtr = malloc(strlen(sig)+1);
			strcpy(*signaturePtr, sig);
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_get_method_location(jclass clazz, jmethodID method,
                                     jlocation *startLocationPtr,
                                     jlocation *endLocationPtr)
{
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*startLocationPtr = 0;
			*endLocationPtr = 1;
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_is_interface(jclass clazz, jboolean *is_interface)
{
	if (clazz == MYCLASS) {
		*is_interface = 0;
		return JVMDI_ERROR_NONE;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}


jvmdiError jvmdi_get_method_modifiers(jclass clazz, jmethodID method,
                                      jint *modifiers)
{
	if (clazz == MYCLASS) {
		if (method == MYMETHOD) {
			*modifiers = 0;
			return JVMDI_ERROR_NONE;
		}
		return JVMDI_ERROR_INVALID_METHODID;
	}
	return JVMDI_ERROR_INVALID_CLASS;
}

jvmdiError jvmdi_get_bytecodes(jclass clazz, jmethodID method,
                               jint *bytecode_count, jbyte **bytecodes)
{
	return JVMDI_ERROR_NOT_IMPLEMENTED;
}

JavaVM create_fake_jvm()
{
	struct JNIInvokeInterface_ *jvm = (struct JNIInvokeInterface_ *)
		malloc(sizeof(struct JNIInvokeInterface_));
	jvm->GetEnv = &jvm_get_env;
	return (JavaVM)jvm;
}


void free_fake_jvm(JavaVM jvm)
{
	free((struct JNIInvokeInterface_ *)jvm);
}


JVMDI_Interface_1 *create_fake_jvmdi()
{
	JVMDI_Interface_1 *iface = malloc(sizeof(JVMDI_Interface_1));
	iface->GetCapabilities = &jvmdi_get_caps;
	iface->SetEventNotificationMode = &jvmdi_set_event_notify;
	iface->SetEventHook = &jvmdi_set_event_hook;
	iface->CreateRawMonitor = &jvmdi_create_raw_monitor;
	iface->DestroyRawMonitor = &jvmdi_destroy_raw_monitor;
	iface->GetClassSignature = &jvmdi_get_class_sig;
	iface->Deallocate = &jvmdi_deallocate;
	iface->RawMonitorEnter = &jvmdi_raw_monitor_enter;
	iface->RawMonitorExit = &jvmdi_raw_monitor_exit;
	iface->GetClassMethods = &jvmdi_get_class_methods;
	iface->IsMethodSynthetic = &jvmdi_is_method_synthetic;
	iface->IsMethodNative = &jvmdi_is_method_native;
	iface->GetSourceFileName = &jvmdi_get_source_file_name;
	iface->GetLineNumberTable = &jvmdi_line_number_table;
	iface->GetMethodName = &jvmdi_get_method_name;
	iface->GetMethodLocation = &jvmdi_get_method_location;
	iface->IsInterface = &jvmdi_is_interface;
	iface->GetMethodModifiers = &jvmdi_get_method_modifiers;
	iface->GetBytecodes = &jvmdi_get_bytecodes;

	return iface;
}


void free_fake_jvmdi(JVMDI_Interface_1 *iface)
{
	free(iface);
}


JNIEnv create_fake_env()
{
	struct JNINativeInterface_ *env = (struct JNINativeInterface_ *)
		malloc(sizeof(struct JNINativeInterface_));

	return (JNIEnv)env;
}


void free_fake_env(JNIEnv env)
{
	free((struct JNINativeInterface_ *)env);
}


void *load_shared_lib()
{
	void *handle;

	handle = dlopen("libcvprof.so", RTLD_NOW);
	if (!handle) {
		fprintf(stderr, "Failed to open profiling library: %s\n", dlerror());
		exit(1);
	}

	return handle;
}

int call_onload_handler(void *handle, JavaVM *jvm, char *opts, void *foo)
{
	int (*on_load)(JavaVM *, char *, void *);
	char *err;

	*(void **) (&on_load) = dlsym(handle, "JVM_OnLoad");
	if ((err = dlerror()) != NULL)  {
		fprintf(stderr, "Couldn't get JVM_OnLoad entry point from profiling library: %s\n", err);
		exit(1);
	}

	return on_load(jvm, opts, foo);
}

void free_shared_lib(void *handle)
{
	dlclose(handle);
}

void do_tests(JNIEnv env)
{
	JVMDI_Event *event = (JVMDI_Event *)malloc(sizeof(JVMDI_Event));


	/* send some test events to cvprof */
	event->kind = JVMDI_EVENT_VM_INIT;
	my_jvmdi_hook(&env, event);

	event->kind = JVMDI_EVENT_CLASS_PREPARE;
	event->u.class_event.thread = NULL;
	event->u.class_event.clazz = MYCLASS;
	my_jvmdi_hook(&env, event);

	event->kind = JVMDI_EVENT_SINGLE_STEP;
	event->u.single_step.thread = NULL;
	event->u.single_step.clazz = MYCLASS;
	event->u.single_step.method = MYMETHOD;
	event->u.single_step.location = (jlocation)0;
	my_jvmdi_hook(&env, event);

	event->kind = JVMDI_EVENT_VM_DEATH;
	my_jvmdi_hook(&env, event);

	free(event);
}

int main(int argc, char **argv)
{
	JavaVM jvm;
	jint err;
	char *opts;
	void *lib_handle = load_shared_lib();
	if (lib_handle == NULL) return 1;

	/* set up a mock implementation of the JVMDI interface */
	my_jvmdi = create_fake_jvmdi();
	jvm = create_fake_jvm();

	opts = "exclude=packages.exclude";
	err = call_onload_handler(lib_handle, &jvm, opts, NULL);
	if (err != JNI_OK) {
		fprintf(stderr, "JVM_OnLoad returned error status %d\n",
		        (int)err);
	} else {
		JNIEnv env = create_fake_env();

		do_tests(env);

		/* clean everything up */
		free_fake_env(env);
	}
	free_fake_jvm(jvm);
	free_fake_jvmdi(my_jvmdi);

	free_shared_lib(lib_handle);

	return 0;
}
