/*
 * exclusions.c
 * $Revision: 1.1 $
 *
 * Copyright (c) 2004 David Holroyd
 * See the file COPYING for license terms
 *
 * http://www.badgers-in-foil.co.uk/projects/cvprof/
 */

#include <stdlib.h>
#include <string.h>
#include "exclusions.h"
#include "err.h"

#define PGK_BUFF_SIZE 1024


int exclusion_list_matches(exclusion_list *list, char *class_signature)
{
	while (list != NULL) {
		char *prefix = list->package_prefix;
		if (!strncmp(class_signature, prefix, strlen(prefix))) {
			return 1;
		}
		list = list->next;
	}
	return 0;
}


void exclusion_list_add(exclusion_list **list, char *class_signature_prefix)
{
	while (*list != NULL) {
		char *prefix = (*list)->package_prefix;
		if (!strncmp(class_signature_prefix, prefix, strlen(prefix))) {
			return;
		}
		list = &(*list)->next;
	}
	(*list) = (exclusion_list *)malloc(sizeof(exclusion_list));
	(*list)->next = NULL;
	(*list)->package_prefix = class_signature_prefix;
}


static void consume_line(FILE *in, int *lines)
{
	char c = EOF;
	while(!feof(in) && (c=fgetc(in))!='\n');
	if (c == '\n') (*lines)++;
}


static char *read_package(FILE *in, int *lines)
{
	char buff[PGK_BUFF_SIZE];
	int count = 0;
	int c;
	char *result;
	int len;

	do {
		while ((c = fgetc(in)) != EOF) {
			if (count == PGK_BUFF_SIZE) {
				EPRINT("Ignored line %d in package list: too long", *lines);
				count = 0;
				consume_line(in, lines);
			} else {
				if (c == '\n') {
					(*lines)++;
					break;
				}
				if (c == '.') c = '/';
				buff[count++] = c;
			}
		}
		buff[count] = 0;

		len = strlen(buff);
	} while (len==0 && !feof(in));

	if (len == 0) return NULL;

	result = (char *)malloc(len+2);
	result[0] = 'L';
	strcpy(result+1, buff);

	return result;
}


exclusion_list *exclusion_list_read(FILE *in)
{
	exclusion_list *list = NULL;
	char *package_prefix;
	int lines = 1;
 
	while ((package_prefix = read_package(in, &lines)) != NULL) {
		exclusion_list_add(&list, package_prefix);
	}

	return list;
}

void exclusion_list_free(exclusion_list *list)
{
	while (list != NULL) {
		exclusion_list *next = list->next;
		free(list->package_prefix);
		free(list);
		list = next;
	}
}
