/*
 * hash.h
 * $Revision: 1.2 $
 *
 * Copyright (c) 2004 David Holroyd
 * See the file COPYING for license terms
 *
 * http://www.badgers-in-foil.co.uk/projects/cvprof/
 *
 */

/*
 * This abstracts the interface to the BerkleyDB hash so that alternative
 * hashtable implementations can be substituted without major changes to
 * cvprof.c
 */

#define DB_DBM_HSEARCH 1
#include <db.h>

/* for O_RDWR O_CREAT */
#include <fcntl.h>

typedef struct DBM *hash_table;


inline hash_table hash_open(char *filename)
{
	return (hash_table)dbm_open(filename, O_RDWR|O_CREAT, 0600);
}


inline void hash_close(hash_table hash)
{
	dbm_close((DBM *)hash);
}


/*
 * Key and value data may be safely changed/freed after calling this function
 * 
 * returns 0 on success
 */
inline int hash_put(hash_table hash, const char *key, int val_size, const void *data)
{
	datum dbm_key;
	datum dbm_value;

	dbm_key.dsize = strlen(key);
	dbm_key.dptr = (void *)key;  /* cast drops 'const' */

	dbm_value.dsize = val_size;
	dbm_value.dptr = (void *)data;  /* cast drops 'const' */

	return dbm_store((DBM *)hash, dbm_key, dbm_value, DBM_REPLACE);
}


/*
 * Key data may be safely changed/freed after calling this function.  Value
 * data is managed by the hash, and will be freed automatically.
 *
 * Returns 0 on success
 */
inline int hash_get(hash_table hash, char *key, int *val_size, void **val_data)
{
	datum dbm_key;
	datum dbm_value;

	dbm_key.dsize = strlen(key);
	dbm_key.dptr = key;

	dbm_value = dbm_fetch((DBM *)hash, dbm_key);

	if(!dbm_value.dsize || !dbm_value.dptr) return 1;

	*val_size = dbm_value.dsize;
	*val_data = dbm_value.dptr;

	return 0;
}
