/*
 * args.c
 * $Revision: 1.1 $
 *
 * Copyright (c) 2004 David Holroyd
 * See the file COPYING for license terms
 *
 * http://www.badgers-in-foil.co.uk/projects/cvprof/
 */


#include <string.h>
#include <stdlib.h>
#include "err.h"
#include "exclusions.h"
#include "options.h"

#define OPT_HELP "help"
#define OPT_EXCLUDE "exclude="

#define OPTS_OK 0
#define OPTS_ERR 1

#define OPT_DELIMITOR ","


static void option_help()
{
	EPRINT("-Xruncvprof[:options] where options may include,\n"
	       "    exclude=<filename> - specify a file listing packages and classes that\n"
	       "                         should not be profiled.\n");
}


static int process_option(options *opts, char *option)
{
	if (*option == 0) {
		/* ignore empty arg */
	} else if(!strcmp(option, OPT_HELP)) {
		option_help();
		return OPTS_ERR;
	} else if(!strncmp(option, OPT_EXCLUDE, strlen(OPT_EXCLUDE))) {
		opts->exclusions_file = strdup(option + strlen(OPT_EXCLUDE));
	} else {
		EPRINT("unknown option \"%s\"", option);
		return OPTS_ERR;
	}

	return OPTS_OK;
}


int options_process_list(options *opts, char *opt_string)
{
	char *opt;
	while ((opt=strsep(&opt_string, OPT_DELIMITOR)) != NULL) {
		if (process_option(opts, opt)) return OPTS_ERR;
	}

	return OPTS_OK;
}

void options_free(options *opts)
{
	if (opts->exclusions_file) free(opts->exclusions_file);
}
